import { GraphQLEnumType, GraphQLEnumValueConfigMap, GraphQLObjectType, GraphQLSchema, GraphQLFieldConfig, GraphQLOutputType, GraphQLFieldResolver, GraphQLArgumentConfig, GraphQLInputType, GraphQLInterfaceType, GraphQLUnionTypeConfig, GraphQLUnionType } from 'graphql';

declare const infostring: (info: InfoString) => {
    name: string;
    description?: string | undefined;
};
declare type InfoString = `${string}${"" | `${";" | "; "}${string}`}`;
declare type DefaultArgs = Record<any, any>;

declare function buildSimpleEnum<E extends Record<string | number | symbol, string | number | symbol>>(info: InfoString, e: E): {
    type: GraphQLEnumType;
    get: (n: keyof E) => string;
} & Record<keyof E, any>;
declare function buildAdvancedEnum(info: InfoString, e: [InfoString, any | undefined][]): {
    type: GraphQLEnumType;
    get: (n: any) => string;
    values: GraphQLEnumValueConfigMap;
};

declare function buildSchema(query: GraphQLObjectType, mutation?: GraphQLObjectType, subscription?: GraphQLObjectType): GraphQLSchema;
declare type BuildQueryFields<Source = any, Context = any, Args = DefaultArgs> = [
    string,
    GraphQLFieldConfig<Source, Context, Args>
];
declare function buildObject<Source = any, Context = any, Args = DefaultArgs>(info: InfoString, fields: TypeFields<Source, Context, Args>): GraphQLObjectType<Source, Context>;

declare type TypeFields<Source, Context, Args = DefaultArgs> = LiteralTypeFields<Source, Context, Args> | FunctionTypeFields<Source, Context, Args>;
declare type LiteralTypeFields<Source, Context, Args = DefaultArgs> = BuildQueryFields<Source, Context, Args>[];
declare type FunctionTypeFields<Source, Context, Args = DefaultArgs> = () => BuildQueryFields<Source, Context, Args>[];
declare function buildField<Source = any, Context = any, Args = DefaultArgs>(info: InfoString, type: GraphQLOutputType, resolve?: GraphQLFieldResolver<Source, Context, Args>, args?: BuildArgument[], subscribe?: GraphQLFieldResolver<Source, Context, Args>): BuildQueryFields<Source, Context, Args>;
declare type BuildArgument = [string, GraphQLArgumentConfig];
declare function buildArgument(info: InfoString, type: GraphQLInputType, defaultValue?: any, deprecated?: string): BuildArgument;

declare function buildInterface<Source = any, Context = any, Args = DefaultArgs>(info: InfoString, fields: TypeFields<Source, Context, Args>): GraphQLInterfaceType;

declare type FilterFn<Context, Args = DefaultArgs> = (rootValue: any, args: Args, context: Context, info: any) => boolean | Promise<boolean>;
declare type ResolverFn<Context, Args = DefaultArgs> = (rootValue: any, args: Args, context: Context, info: any) => AsyncIterator<any>;
declare function withTypedFilter<Context, Args = DefaultArgs>(iterator: ResolverFn<Context, Args>, filter: FilterFn<Context, Args>): ResolverFn<Context, Args>;

declare function buildUnion<Source = any, Context = any>(info: InfoString, types: GraphQLUnionTypeConfig<Source, Context>["types"]): GraphQLUnionType;

declare const all_buildSimpleEnum: typeof buildSimpleEnum;
declare const all_buildAdvancedEnum: typeof buildAdvancedEnum;
type all_TypeFields<_0, _1, _2> = TypeFields<_0, _1, _2>;
type all_LiteralTypeFields<_0, _1, _2> = LiteralTypeFields<_0, _1, _2>;
type all_FunctionTypeFields<_0, _1, _2> = FunctionTypeFields<_0, _1, _2>;
declare const all_buildField: typeof buildField;
type all_BuildArgument = BuildArgument;
declare const all_buildArgument: typeof buildArgument;
declare const all_buildInterface: typeof buildInterface;
declare const all_buildSchema: typeof buildSchema;
type all_BuildQueryFields<_0, _1, _2> = BuildQueryFields<_0, _1, _2>;
declare const all_buildObject: typeof buildObject;
type all_FilterFn<_0, _1> = FilterFn<_0, _1>;
type all_ResolverFn<_0, _1> = ResolverFn<_0, _1>;
declare const all_withTypedFilter: typeof withTypedFilter;
declare const all_buildUnion: typeof buildUnion;
declare const all_infostring: typeof infostring;
type all_InfoString = InfoString;
type all_DefaultArgs = DefaultArgs;
declare namespace all {
  export {
    all_buildSimpleEnum as buildSimpleEnum,
    all_buildAdvancedEnum as buildAdvancedEnum,
    all_TypeFields as TypeFields,
    all_LiteralTypeFields as LiteralTypeFields,
    all_FunctionTypeFields as FunctionTypeFields,
    all_buildField as buildField,
    all_BuildArgument as BuildArgument,
    all_buildArgument as buildArgument,
    all_buildInterface as buildInterface,
    all_buildSchema as buildSchema,
    all_BuildQueryFields as BuildQueryFields,
    all_buildObject as buildObject,
    all_FilterFn as FilterFn,
    all_ResolverFn as ResolverFn,
    all_withTypedFilter as withTypedFilter,
    all_buildUnion as buildUnion,
    all_infostring as infostring,
    all_InfoString as InfoString,
    all_DefaultArgs as DefaultArgs,
  };
}

export { BuildArgument, BuildQueryFields, DefaultArgs, FilterFn, FunctionTypeFields, InfoString, LiteralTypeFields, ResolverFn, TypeFields, buildAdvancedEnum, buildArgument, buildField, buildInterface, buildObject, buildSchema, buildSimpleEnum, buildUnion, all as e, infostring, withTypedFilter };
